@@ -1,22 +1,19 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * An abstract class which defines the API to a given backend.
- * There will be one EBookBackend object for every URI which is loaded.
+ * e-book-backend.h
  *
- * Two people will call into the EBookBackend API:
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) version 3.
  *
- * 1. The PASBookFactory, when it has been asked to load a book.
- *    It will create a new EBookBackend if one is not already running
- *    for the requested URI.  It will call e_book_backend_add_client to
- *    add a new client to an existing EBookBackend server.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * 2. A PASBook, when a client has requested an operation on the
- *    GNOME_Evolution_Addressbook_Book interface.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with the program; if not, see <http://www.gnu.org/licenses/>
  *
- * Author:
- *   Nat Friedman (nat@ximian.com)
- *
- * Copyright (C) 1999-2008 Novell, Inc. (www.novell.com)
  */
 
 #if !defined (__LIBEDATA_BOOK_H_INSIDE__) && !defined (LIBEDATA_BOOK_COMPILATION)
@@ -52,33 +49,6 @@
 	(G_TYPE_INSTANCE_GET_CLASS \
 	((obj), E_TYPE_BOOK_BACKEND, EBookBackendClass))
 
-/**
- * CLIENT_BACKEND_PROPERTY_ONLINE:
- *
- * FIXME: Document me.
- *
- * Since: 3.2
- **/
-#define CLIENT_BACKEND_PROPERTY_ONLINE			"online"
-
-/**
- * CLIENT_BACKEND_PROPERTY_READONLY:
- *
- * FIXME: Document me.
- *
- * Since: 3.2
- **/
-#define CLIENT_BACKEND_PROPERTY_READONLY		"readonly"
-
-/**
- * CLIENT_BACKEND_PROPERTY_CACHE_DIR:
- *
- * FIXME: Document me.
- *
- * Since: 3.2
- **/
-#define CLIENT_BACKEND_PROPERTY_CACHE_DIR		"cache-dir"
-
 /**
  * CLIENT_BACKEND_PROPERTY_CAPABILITIES:
  *
@@ -131,27 +101,65 @@ struct _EBookBackend {
 struct _EBookBackendClass {
 	EBackendClass parent_class;
 
-	/* Virtual methods */
-	void		(*get_backend_property)	(EBookBackend *backend,
-						 EDataBook *book,
-						 guint32 opid,
-						 GCancellable *cancellable,
+	/* Set this to TRUE to use a serial dispatch queue, instead
+	 * of a concurrent dispatch queue.  A serial dispatch queue
+	 * executes one method at a time in the order in which they
+	 * were called.  This is generally slower than a concurrent
+	 * dispatch queue, but helps avoid thread-safety issues. */
+	gboolean use_serial_dispatch_queue;
+
+	gchar *		(*get_backend_property)	(EBookBackend *backend,
 						 const gchar *prop_name);
 
-	/* This method is deprecated. */
-	void		(*set_backend_property)	(EBookBackend *backend,
-						 EDataBook *book,
-						 guint32 opid,
+	gboolean	(*open_sync)		(EBookBackend *backend,
 						 GCancellable *cancellable,
-						 const gchar *prop_name,
-						 const gchar *prop_value);
+						 GError **error);
+	gboolean	(*refresh_sync)		(EBookBackend *backend,
+						 GCancellable *cancellable,
+						 GError **error);
+	gboolean	(*create_contacts_sync)	(EBookBackend *backend,
+						 const gchar * const *vcards,
+						 GQueue *out_contacts,
+						 GCancellable *cancellable,
+						 GError **error);
+	gboolean	(*modify_contacts_sync)	(EBookBackend *backend,
+						 const gchar * const *vcards,
+						 GQueue *out_contacts,
+						 GCancellable *cancellable,
+						 GError **error);
+	gboolean	(*remove_contacts_sync)	(EBookBackend *backend,
+						 const gchar * const *uids,
+						 GCancellable *cancellable,
+						 GError **error);
+	EContact *	(*get_contact_sync)	(EBookBackend *backend,
+						 const gchar *uid,
+						 GCancellable *cancellable,
+						 GError **error);
+	gboolean	(*get_contact_list_sync)
+						(EBookBackend *backend,
+						 const gchar *query,
+						 GQueue *out_contacts,
+						 GCancellable *cancellable,
+						 GError **error);
 
+	/* This method is optional.  By default, it simply calls
+	 * get_contact_list_sync() and extracts UID strings from
+	 * the matched EContacts.  Backends may override this if
+	 * they can implement it more efficiently. */
+	gboolean	(*get_contact_list_uids_sync)
+						(EBookBackend *backend,
+						 const gchar *query,
+						 GQueue *out_uids,
+						 GCancellable *cancellable,
+						 GError **error);
+
+	/* These methods are deprecated and will be removed once all
+	 * known subclasses are converted to the new methods above. */
 	void		(*open)			(EBookBackend *backend,
 						 EDataBook *book,
 						 guint32 opid,
 						 GCancellable *cancellable,
 						 gboolean only_if_exists);
-
 	void		(*refresh)		(EBookBackend *backend,
 						 EDataBook *book,
 						 guint32 opid,
@@ -196,82 +204,149 @@ struct _EBookBackendClass {
 	void		(*notify_update)	(EBookBackend *backend,
 						 const EContact *contact);
 
-
 	EDataBookDirect *
-	        (* get_direct_book)             (EBookBackend *backend);
-	void    (* configure_direct)            (EBookBackend *backend, const gchar *config);
+			(*get_direct_book)	(EBookBackend *backend);
+	void		(*configure_direct)	(EBookBackend *backend,
+						 const gchar *config);
 
-	/* Notification signals */
 	void		(*sync)			(EBookBackend *backend);
+
+	/* Signals */
+	void		(*closed)		(EBookBackend *backend,
+						 const gchar *sender);
 };
 
 GType		e_book_backend_get_type		(void) G_GNUC_CONST;
 
 const gchar *	e_book_backend_get_cache_dir	(EBookBackend *backend);
+gchar *		e_book_backend_dup_cache_dir	(EBookBackend *backend);
 void		e_book_backend_set_cache_dir	(EBookBackend *backend,
 						 const gchar *cache_dir);
+EDataBook *	e_book_backend_ref_data_book	(EBookBackend *backend);
+void		e_book_backend_set_data_book	(EBookBackend *backend,
+						 EDataBook *data_book);
 ESourceRegistry *
 		e_book_backend_get_registry	(EBookBackend *backend);
 gboolean	e_book_backend_get_writable	(EBookBackend *backend);
 void		e_book_backend_set_writable	(EBookBackend *backend,
 						 gboolean writable);
 
-gboolean	e_book_backend_add_client	(EBookBackend *backend,
-						 EDataBook *book);
-void		e_book_backend_remove_client	(EBookBackend *backend,
-						 EDataBook *book);
-
 gboolean	e_book_backend_is_opened	(EBookBackend *backend);
 gboolean	e_book_backend_is_readonly	(EBookBackend *backend);
-gboolean	e_book_backend_is_removed	(EBookBackend *backend);
 
-void		e_book_backend_get_backend_property
+gchar *		e_book_backend_get_backend_property
 						(EBookBackend *backend,
-						 EDataBook *book,
-						 guint32 opid,
-						 GCancellable *cancellable,
 						 const gchar *prop_name);
-
+gboolean	e_book_backend_open_sync	(EBookBackend *backend,
+						 GCancellable *cancellable,
+						 GError **error);
 void		e_book_backend_open		(EBookBackend *backend,
-						 EDataBook *book,
-						 guint32 opid,
 						 GCancellable *cancellable,
-						 gboolean only_if_exists);
+						 GAsyncReadyCallback callback,
+						 gpointer user_data);
+gboolean	e_book_backend_open_finish	(EBookBackend *backend,
+						 GAsyncResult *result,
+						 GError **error);
+gboolean	e_book_backend_refresh_sync	(EBookBackend *backend,
+						 GCancellable *cancellable,
+						 GError **error);
 void		e_book_backend_refresh		(EBookBackend *backend,
-						 EDataBook *book,
-						 guint32 opid,
-						 GCancellable *cancellable);
+						 GCancellable *cancellable,
+						 GAsyncReadyCallback callback,
+						 gpointer user_data);
+gboolean	e_book_backend_refresh_finish	(EBookBackend *backend,
+						 GAsyncResult *result,
+						 GError **error);
+gboolean	e_book_backend_create_contacts_sync
+						(EBookBackend *backend,
+						 const gchar * const *vcards,
+						 GQueue *out_contacts,
+						 GCancellable *cancellable,
+						 GError **error);
 void		e_book_backend_create_contacts	(EBookBackend *backend,
-						 EDataBook *book,
-						 guint32 opid,
+						 const gchar * const *vcards,
 						 GCancellable *cancellable,
-						 const GSList *vcards);
-void		e_book_backend_remove_contacts	(EBookBackend *backend,
-						 EDataBook *book,
-						 guint32 opid,
+						 GAsyncReadyCallback callback,
+						 gpointer user_data);
+gboolean	e_book_backend_create_contacts_finish
+						(EBookBackend *backend,
+						 GAsyncResult *result,
+						 GQueue *out_contacts,
+						 GError **error);
+gboolean	e_book_backend_modify_contacts_sync
+						(EBookBackend *backend,
+						 const gchar * const *vcards,
 						 GCancellable *cancellable,
-						 const GSList *id_list);
+						 GError **error);
 void		e_book_backend_modify_contacts	(EBookBackend *backend,
-						 EDataBook *book,
-						 guint32 opid,
+						 const gchar * const *vcards,
 						 GCancellable *cancellable,
-						 const GSList *vcards);
+						 GAsyncReadyCallback callback,
+						 gpointer user_data);
+gboolean	e_book_backend_modify_contacts_finish
+						(EBookBackend *backend,
+						 GAsyncResult *result,
+						 GError **error);
+gboolean	e_book_backend_remove_contacts_sync
+						(EBookBackend *backend,
+						 const gchar * const *uids,
+						 GCancellable *cancellable,
+						 GError **error);
+void		e_book_backend_remove_contacts	(EBookBackend *backend,
+						 const gchar * const *uids,
+						 GCancellable *cancellable,
+						 GAsyncReadyCallback callback,
+						 gpointer user_data);
+gboolean	e_book_backend_remove_contacts_finish
+						(EBookBackend *backend,
+						 GAsyncResult *result,
+						 GError **error);
+EContact *	e_book_backend_get_contact_sync	(EBookBackend *backend,
+						 const gchar *uid,
+						 GCancellable *cancellable,
+						 GError **error);
 void		e_book_backend_get_contact	(EBookBackend *backend,
-						 EDataBook *book,
-						 guint32 opid,
+						 const gchar *uid,
 						 GCancellable *cancellable,
-						 const gchar *id);
+						 GAsyncReadyCallback callback,
+						 gpointer user_data);
+EContact *	e_book_backend_get_contact_finish
+						(EBookBackend *backend,
+						 GAsyncResult *result,
+						 GError **error);
+gboolean	e_book_backend_get_contact_list_sync
+						(EBookBackend *backend,
+						 const gchar *query,
+						 GQueue *out_contacts,
+						 GCancellable *cancellable,
+						 GError **error);
 void		e_book_backend_get_contact_list	(EBookBackend *backend,
-						 EDataBook *book,
-						 guint32 opid,
+						 const gchar *query,
 						 GCancellable *cancellable,
-						 const gchar *query);
+						 GAsyncReadyCallback callback,
+						 gpointer user_data);
+gboolean	e_book_backend_get_contact_list_finish
+						(EBookBackend *backend,
+						 GAsyncResult *result,
+						 GQueue *out_contacts,
+						 GError **error);
+gboolean	e_book_backend_get_contact_list_uids_sync
+						(EBookBackend *backend,
+						 const gchar *query,
+						 GQueue *out_uids,
+						 GCancellable *cancellable,
+						 GError **error);
 void		e_book_backend_get_contact_list_uids
 						(EBookBackend *backend,
-						 EDataBook *book,
-						 guint32 opid,
+						 const gchar *query,
 						 GCancellable *cancellable,
-						 const gchar *query);
+						 GAsyncReadyCallback callback,
+						 gpointer user_data);
+gboolean	e_book_backend_get_contact_list_uids_finish
+						(EBookBackend *backend,
+						 GAsyncResult *result,
+						 GQueue *out_uids,
+						 GError **error);
 
 void		e_book_backend_start_view	(EBookBackend *backend,
 						 EDataBookView *view);
@@ -297,72 +372,17 @@ void		e_book_backend_notify_property_changed
 						 const gchar *prop_value);
 
 EDataBookDirect *
-                e_book_backend_get_direct_book  (EBookBackend *backend);
-void            e_book_backend_configure_direct (EBookBackend *backend, const gchar *config);
+		e_book_backend_get_direct_book	(EBookBackend *backend);
+void		e_book_backend_configure_direct	(EBookBackend *backend,
+						 const gchar *config);
 
 void		e_book_backend_sync		(EBookBackend *backend);
 
-/* protected functions for subclasses */
-void		e_book_backend_set_is_removed	(EBookBackend *backend,
-						 gboolean is_removed);
-
-#ifndef EDS_DISABLE_DEPRECATED
-/**
- * CLIENT_BACKEND_PROPERTY_OPENED:
- *
- * FIXME: Document me.
- *
- * Since: 3.2
- *
- * Deprecated: 3.8: Clients don't need to care if they're fully opened
- *                  anymore.  This property will always return %TRUE.
- **/
-#define CLIENT_BACKEND_PROPERTY_OPENED			"opened"
-
-/**
- * CLIENT_BACKEND_PROPERTY_OPENING:
- *
- * FIXME: Document me.
- *
- * Since: 3.2
- *
- * Deprecated: 3.8: Clients don't need to care if they're fully opened
- *                  anymore.  This property will always return %FALSE.
- **/
-#define CLIENT_BACKEND_PROPERTY_OPENING			"opening"
-
-/**
- * BOOK_BACKEND_PROPERTY_SUPPORTED_AUTH_METHODS:
- *
- * Since: 3.2
- *
- * Deprecated: 3.8: The property is no longer supported.
- **/
-#define BOOK_BACKEND_PROPERTY_SUPPORTED_AUTH_METHODS	"supported-auth-methods"
-
-gboolean	e_book_backend_is_opening	(EBookBackend *backend);
-void		e_book_backend_set_backend_property
+GSimpleAsyncResult *
+		e_book_backend_prepare_for_completion
 						(EBookBackend *backend,
-						 EDataBook *book,
 						 guint32 opid,
-						 GCancellable *cancellable,
-						 const gchar *prop_name,
-						 const gchar *prop_value);
-void		e_book_backend_foreach_view	(EBookBackend *backend,
-						 gboolean (*callback) (EDataBookView *view,
-								       gpointer user_data),
-								       gpointer user_data);
-void		e_book_backend_notify_opened	(EBookBackend *backend,
-						 GError *error);
-void		e_book_backend_notify_readonly	(EBookBackend *backend,
-						 gboolean is_readonly);
-void		e_book_backend_notify_online	(EBookBackend *backend,
-						 gboolean is_online);
-void		e_book_backend_respond_opened	(EBookBackend *backend,
-						 EDataBook *book,
-						 guint32 opid,
-						 GError *error);
-#endif /* EDS_DISABLE_DEPRECATED */
+						 GQueue **result_queue);
 
 G_END_DECLS
 
