@@ -1026,14 +1026,20 @@ e_cal_backend_http_get_object (ECalBackendSync *backend,
 		return;
 	}
 
-	comp = e_cal_backend_store_get_component (priv->store, uid, rid);
-	if (!comp) {
-		g_propagate_error (error, EDC_ERROR (ObjectNotFound));
-		return;
-	}
+	if (rid && *rid) {
+		comp = e_cal_backend_store_get_component (priv->store, uid, rid);
+		if (!comp) {
+			g_propagate_error (error, EDC_ERROR (ObjectNotFound));
+			return;
+		}
 
-	*object = e_cal_component_get_as_string (comp);
-	g_object_unref (comp);
+		*object = e_cal_component_get_as_string (comp);
+		g_object_unref (comp);
+	} else {
+		*object = e_cal_backend_store_get_components_by_uid_as_ical_string (priv->store, uid);
+		if (!*object)
+			g_propagate_error (error, EDC_ERROR (ObjectNotFound));
+	}
 }
 
 /* Add_timezone handler for the file backend */
