@@ -156,7 +156,6 @@ struct _CamelIMAPXServer {
 
 	/* Idle */
 	CamelIMAPXIdle *idle;
-	gboolean use_idle;
 
 	gboolean use_qresync;
 };
@@ -164,11 +163,6 @@ struct _CamelIMAPXServer {
 struct _CamelIMAPXServerClass {
 	CamelObjectClass parent_class;
 
-	/* Signals */
-	void	(*select_changed)	(CamelIMAPXServer *is,
-					 const gchar *selected_folder);
-	void	(*shutdown)		(CamelIMAPXServer *is);
-
 	gchar tagprefix;
 };
 
@@ -194,7 +188,7 @@ CamelAuthenticationResult
 						 GError **error);
 GPtrArray *	camel_imapx_server_list		(CamelIMAPXServer *is,
 						 const gchar *top,
-						 guint32 flags,
+						 CamelStoreGetFolderInfoFlags flags,
 						 const gchar *ext,
 						 GCancellable *cancellable,
 						 GError **error);
