@@ -43,16 +43,16 @@
 #define k(x) /*(printf ("%s (%d):%s: ",  __FILE__, __LINE__, __PRETTY_FUNCTION__),(x))*/
 
 #define CAMEL_PARTITION_TABLE_LOCK(kf, lock) \
-	(g_static_mutex_lock (&(kf)->priv->lock))
+	(g_mutex_lock (&(kf)->priv->lock))
 #define CAMEL_PARTITION_TABLE_UNLOCK(kf, lock) \
-	(g_static_mutex_unlock (&(kf)->priv->lock))
+	(g_mutex_unlock (&(kf)->priv->lock))
 
 #define CAMEL_PARTITION_TABLE_GET_PRIVATE(obj) \
 	(G_TYPE_INSTANCE_GET_PRIVATE \
 	((obj), CAMEL_TYPE_PARTITION_TABLE, CamelPartitionTablePrivate))
 
 struct _CamelPartitionTablePrivate {
-	GStaticMutex lock;	/* for locking partition */
+	GMutex lock;	/* for locking partition */
 };
 
 G_DEFINE_TYPE (CamelPartitionTable, camel_partition_table, CAMEL_TYPE_OBJECT)
@@ -73,7 +73,7 @@ partition_table_finalize (GObject *object)
 		g_object_unref (table->blocks);
 	}
 
-	g_static_mutex_free (&table->priv->lock);
+	g_mutex_clear (&table->priv->lock);
 
 	/* Chain up to parent's finalize() method. */
 	G_OBJECT_CLASS (camel_partition_table_parent_class)->finalize (object);
@@ -96,7 +96,7 @@ camel_partition_table_init (CamelPartitionTable *cpi)
 	cpi->priv = CAMEL_PARTITION_TABLE_GET_PRIVATE (cpi);
 
 	g_queue_init (&cpi->partition);
-	g_static_mutex_init (&cpi->priv->lock);
+	g_mutex_init (&cpi->priv->lock);
 }
 
 /* ********************************************************************** */
@@ -622,12 +622,12 @@ fail:
 	((obj), CAMEL_TYPE_KEY_TABLE, CamelKeyTablePrivate))
 
 #define CAMEL_KEY_TABLE_LOCK(kf, lock) \
-	(g_static_mutex_lock (&(kf)->priv->lock))
+	(g_mutex_lock (&(kf)->priv->lock))
 #define CAMEL_KEY_TABLE_UNLOCK(kf, lock) \
-	(g_static_mutex_unlock (&(kf)->priv->lock))
+	(g_mutex_unlock (&(kf)->priv->lock))
 
 struct _CamelKeyTablePrivate {
-	GStaticMutex lock;	/* for locking key */
+	GMutex lock;	/* for locking key */
 };
 
 G_DEFINE_TYPE (CamelKeyTable, camel_key_table, CAMEL_TYPE_OBJECT)
@@ -646,7 +646,7 @@ key_table_finalize (GObject *object)
 		g_object_unref (table->blocks);
 	}
 
-	g_static_mutex_free (&table->priv->lock);
+	g_mutex_clear (&table->priv->lock);
 
 	/* Chain up to parent's finalize() method. */
 	G_OBJECT_CLASS (camel_key_table_parent_class)->finalize (object);
@@ -667,7 +667,7 @@ static void
 camel_key_table_init (CamelKeyTable *table)
 {
 	table->priv = CAMEL_KEY_TABLE_GET_PRIVATE (table);
-	g_static_mutex_init (&table->priv->lock);
+	g_mutex_init (&table->priv->lock);
 }
 
 CamelKeyTable *
