@@ -76,7 +76,7 @@ sasl_popb4smtp_challenge_sync (CamelSasl *sasl,
 	gchar *pop_uid;
 
 	service = camel_sasl_get_service (sasl);
-	session = camel_service_get_session (service);
+	session = camel_service_ref_session (service);
 
 	camel_sasl_set_authenticated (sasl, FALSE);
 
@@ -89,6 +89,8 @@ sasl_popb4smtp_challenge_sync (CamelSasl *sasl,
 	else
 		service = NULL;
 
+	g_object_unref (session);
+
 	if (service == NULL) {
 		g_set_error (
 			error, CAMEL_SERVICE_ERROR,
