@@ -44,10 +44,15 @@ main (gint argc,
 	g_type_init ();
 #endif
 	g_test_init (&argc, &argv, NULL);
+	g_test_bug_base ("http://bugzilla.gnome.org/");
 
 	g_test_add (
-		"/ECal/CreateObject", ETestServerFixture, &cal_closure,
-		e_test_server_utils_setup, test_create_object, e_test_server_utils_teardown);
+		"/ECal/CreateObject",
+		ETestServerFixture,
+		&cal_closure,
+		e_test_server_utils_setup,
+		test_create_object,
+		e_test_server_utils_teardown);
 
 	return e_test_server_utils_run ();
 }
