@@ -51,10 +51,15 @@ main (gint argc,
 	g_type_init ();
 #endif
 	g_test_init (&argc, &argv, NULL);
+	g_test_bug_base ("http://bugzilla.gnome.org/");
 
 	g_test_add (
-		"/ECal/SendObjects", ETestServerFixture, &cal_closure,
-		e_test_server_utils_setup, test_send_objects, e_test_server_utils_teardown);
+		"/ECal/SendObjects",
+		ETestServerFixture,
+		&cal_closure,
+		e_test_server_utils_setup,
+		test_send_objects,
+		e_test_server_utils_teardown);
 
 	return e_test_server_utils_run ();
 }
